"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Code = exports.codeType = exports.code = void 0;
const tslib_1 = require("tslib");
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_codemark_1 = tslib_1.__importDefault(require("prosemirror-codemark"));
const prosemirror_state_1 = require("prosemirror-state");
const actions_1 = require("../../../utils/actions");
const schema_1 = require("../../../utils/schema");
exports.code = 'code';
const codeAction = 'code';
exports.codeType = (0, schema_1.markTypeFactory)(exports.code);
const Code = (builder, opts) => {
    builder
        .addMark(exports.code, () => ({
        spec: {
            code: true,
            parseDOM: [{ tag: 'code' }],
            toDOM() {
                return ['code'];
            },
        },
        toYfm: {
            open(_state, _mark, parent, index) {
                return backticksFor(parent.child(index), -1);
            },
            close(_state, _mark, parent, index) {
                return backticksFor(parent.child(index - 1), 1);
            },
            escape: false,
        },
        fromYfm: {
            tokenSpec: { name: exports.code, type: 'mark', noCloseToken: true },
            tokenName: 'code_inline',
        },
    }), builder.Priority.Lowest)
        .addAction(codeAction, ({ schema }) => (0, actions_1.createToggleMarkAction)((0, exports.codeType)(schema)));
    if (opts === null || opts === void 0 ? void 0 : opts.codeKey) {
        const { codeKey } = opts;
        builder.addKeymap(({ schema }) => ({ [codeKey]: (0, prosemirror_commands_1.toggleMark)((0, exports.codeType)(schema)) }));
    }
    builder
        // codemark adds inputRule for `code`,
        // adds fake cursor behaviour when crossing inline_code boundaries,
        // wrap current selected text to inline_code when '`' was pressed.
        // See demo: https://curvenote.github.io/prosemirror-codemark/
        .addPlugin(({ schema }) => (0, prosemirror_codemark_1.default)({ markType: (0, exports.codeType)(schema) }));
    builder.addPlugin(() => 
    // apply codemark when typing text between ``
    new prosemirror_state_1.Plugin({
        props: {
            handleTextInput: (view, from, to, text) => {
                var _a, _b, _c, _d;
                const { selection: { $anchor }, tr, schema, } = view.state;
                if (((_b = (_a = $anchor.nodeBefore) === null || _a === void 0 ? void 0 : _a.text) === null || _b === void 0 ? void 0 : _b.endsWith('`')) &&
                    ((_d = (_c = $anchor.nodeAfter) === null || _c === void 0 ? void 0 : _c.text) === null || _d === void 0 ? void 0 : _d.startsWith('`'))) {
                    view.dispatch(tr.replaceRangeWith(from - 1, to + 1, view.state.schema.text(text, [(0, exports.codeType)(schema).create()])));
                    return true;
                }
                return false;
            },
        },
    }));
};
exports.Code = Code;
function backticksFor(node, side) {
    const ticks = /`+/g;
    let m;
    let len = 0;
    if (node.isText)
        while ((m = ticks.exec(node.text || '')))
            len = Math.max(len, m[0].length);
    let result = len > 0 && side > 0 ? ' `' : '`';
    for (let i = 0; i < len; i++)
        result += '`';
    if (len > 0 && side < 0)
        result += ' ';
    return result;
}
