"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HorizontalRule = exports.markupAttr = exports.horizontalRule = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const schema_1 = require("../../../utils/schema");
const inputrules_1 = require("../../../utils/inputrules");
const BaseSchema_1 = require("../../base/BaseSchema");
exports.horizontalRule = 'horizontal_rule';
const hrAction = 'hRule';
exports.markupAttr = 'markup';
const hrType = (0, schema_1.nodeTypeFactory)(exports.horizontalRule);
const HorizontalRule = (builder) => {
    builder.addNode(exports.horizontalRule, () => ({
        spec: {
            attrs: { [exports.markupAttr]: { default: '---' } },
            group: 'block',
            parseDOM: [{ tag: 'hr' }],
            toDOM() {
                return ['div', ['hr']];
            },
        },
        fromYfm: {
            tokenName: 'hr',
            tokenSpec: {
                name: exports.horizontalRule,
                type: 'node',
                getAttrs: (token) => ({ [exports.markupAttr]: token.markup }),
            },
        },
        toYfm: (state, node) => {
            state.write(node.attrs[exports.markupAttr]);
            state.closeBlock(node);
        },
    }));
    builder.addInputRules((deps) => ({
        rules: [
            // --- or ___ or *** at start of line are converted to horizontal line
            // and add new empty paragraph after
            (0, inputrules_1.nodeInputRule)(/^(---|___|\*\*\*)$/, (markup) => [
                hrType(deps.schema).create({ [exports.markupAttr]: markup }),
                (0, BaseSchema_1.pType)(deps.schema).create(),
            ], 1),
        ],
    }));
    builder.addAction(hrAction, ({ schema }) => {
        const cmd = addHr(hrType(schema));
        return {
            isActive: (state) => isHrSelection(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
};
exports.HorizontalRule = HorizontalRule;
const addHr = (hr) => (state, dispatch) => {
    if (!isHrSelection(state.selection)) {
        dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.replaceSelectionWith(hr.create()).scrollIntoView());
    }
    return true;
};
function isHrSelection(selection) {
    return selection instanceof prosemirror_state_1.NodeSelection && selection.node.type.name === exports.horizontalRule;
}
