"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Italic = exports.italic = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const actions_1 = require("../../../utils/actions");
const schema_1 = require("../../../utils/schema");
const inputrules_1 = require("../../../utils/inputrules");
exports.italic = 'em';
const iAction = 'italic';
const iType = (0, schema_1.markTypeFactory)(exports.italic);
const Italic = (builder, opts) => {
    builder
        .addMark(exports.italic, () => ({
        spec: {
            parseDOM: [
                { tag: 'i' },
                { tag: 'em' },
                { style: 'font-style', getAttrs: (value) => value === 'italic' && null },
            ],
            toDOM() {
                return ['em'];
            },
        },
        toYfm: { open: '*', close: '*', mixable: true, expelEnclosingWhitespace: true },
        fromYfm: { tokenSpec: { name: exports.italic, type: 'mark' } },
    }))
        .addAction(iAction, ({ schema }) => (0, actions_1.createToggleMarkAction)(iType(schema)));
    builder.addInputRules(({ schema }) => ({
        rules: [
            (0, inputrules_1.markInputRule)({ open: '*', close: '*', ignoreBetween: '*' }, iType(schema)),
            (0, inputrules_1.markInputRule)({ open: '_', close: '_', ignoreBetween: '_' }, iType(schema)),
        ],
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.italicKey) {
        const { italicKey } = opts;
        builder.addKeymap(({ schema }) => ({ [italicKey]: (0, prosemirror_commands_1.toggleMark)(iType(schema)) }));
    }
};
exports.Italic = Italic;
