"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.moveTextblockToEndOfLastItemOfPrevList = exports.liftIfCursorIsAtBeginningOfItem = exports.toList = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_schema_list_1 = require("prosemirror-schema-list");
const utils_1 = require("./utils");
const selection_1 = require("../../../utils/selection");
const schema_1 = require("../../../utils/schema");
function toList(listType) {
    return (state, dispatch) => {
        const parentList = (0, utils_1.findAnyParentList)(state.schema)(state.selection);
        if (parentList) {
            if (listType === parentList.node.type)
                return false;
            dispatch === null || dispatch === void 0 ? void 0 : dispatch(state.tr.setNodeMarkup(parentList.pos, listType));
            return true;
        }
        return (0, prosemirror_schema_list_1.wrapInList)(listType)(state, dispatch);
    };
}
exports.toList = toList;
const liftIfCursorIsAtBeginningOfItem = (state, dispatch, view) => {
    const $cursor = (0, selection_1.get$Cursor)(state.selection);
    if (!$cursor)
        return false;
    const { schema } = state;
    const parentBlock = $cursor.node($cursor.depth - 1);
    if ((view === null || view === void 0 ? void 0 : view.endOfTextblock('backward', state)) &&
        parentBlock.firstChild === $cursor.parent &&
        (0, schema_1.isSameNodeType)(parentBlock, (0, utils_1.liType)(schema))) {
        return (0, prosemirror_schema_list_1.liftListItem)((0, utils_1.liType)(schema))(state, dispatch);
    }
    return false;
};
exports.liftIfCursorIsAtBeginningOfItem = liftIfCursorIsAtBeginningOfItem;
const moveTextblockToEndOfLastItemOfPrevList = (state, dispatch, view) => {
    const $cursor = (0, selection_1.get$Cursor)(state.selection);
    if (!$cursor)
        return false;
    const { schema } = state;
    if (view === null || view === void 0 ? void 0 : view.endOfTextblock('backward', state)) {
        const parentBlock = $cursor.node($cursor.depth - 1);
        const currentTextblock = findNodeInParent($cursor.parent, parentBlock);
        if (!currentTextblock)
            return false;
        const maybeList = parentBlock.maybeChild(currentTextblock.index - 1);
        if (maybeList &&
            ((0, schema_1.isSameNodeType)(maybeList, (0, utils_1.blType)(schema)) || (0, schema_1.isSameNodeType)(maybeList, (0, utils_1.olType)(schema)))) {
            if (dispatch) {
                const from = $cursor.before();
                const to = $cursor.after();
                const tr = state.tr.delete(from, to).insert(from - 2, $cursor.parent);
                dispatch(tr.setSelection(prosemirror_state_1.TextSelection.create(tr.doc, from - 1)));
            }
            return true;
        }
    }
    return false;
};
exports.moveTextblockToEndOfLastItemOfPrevList = moveTextblockToEndOfLastItemOfPrevList;
function findNodeInParent(node, parent) {
    let index = -1;
    let offset = 0;
    parent.forEach((n, o, i) => {
        if (node === n) {
            offset = o;
            index = i;
        }
    });
    if (index < 0)
        return null;
    return { node, offset, index };
}
