"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tabBackspace = exports.tabPanelBackspace = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_utils_1 = require("prosemirror-utils");
const const_1 = require("./const");
const helpers_1 = require("../../../table-utils/helpers");
const selection_1 = require("../../../utils/selection");
const tabPanelBackspace = (state) => {
    const $cursor = (0, selection_1.get$Cursor)(state.selection);
    if (($cursor === null || $cursor === void 0 ? void 0 : $cursor.node($cursor.depth - 1).type) === (0, const_1.tabPanelType)(state.schema) &&
        $cursor.start($cursor.depth - 1) === $cursor.pos - 1) {
        return true;
    }
    return false;
};
exports.tabPanelBackspace = tabPanelBackspace;
const tabBackspace = (state, dispatch) => {
    const tabToRemove = (0, prosemirror_utils_1.findParentNodeOfType)((0, const_1.tabType)(state.schema))(state.selection);
    const tabsParentNode = (0, prosemirror_utils_1.findParentNodeOfType)((0, const_1.tabsType)(state.schema))(state.selection);
    if (tabsParentNode &&
        tabToRemove &&
        state.selection.from === tabToRemove.pos + 1 &&
        state.selection.from === state.selection.to) {
        const tabList = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (tabNode) => {
            return tabNode.type.name === (0, const_1.tabListType)(state.schema).name;
        })[0];
        const tabToRemoveIdx = (0, helpers_1.findChildIndex)(tabList.node, tabToRemove.node);
        const tabNodes = (0, prosemirror_utils_1.findChildren)(tabList.node, (node) => node.type.name === (0, const_1.tabType)(state.schema).name);
        const tabPanels = (0, prosemirror_utils_1.findChildren)(tabsParentNode.node, (tabNode) => {
            return tabNode.type.name === (0, const_1.tabPanelType)(state.schema).name;
        });
        const panelToRemove = tabPanels.filter((tabNode) => tabNode.node.attrs['aria-labelledby'] === tabToRemove.node.attrs['id'])[0];
        if (panelToRemove && dispatch) {
            // Change relative pos to absolute
            panelToRemove.pos = panelToRemove.pos + tabsParentNode.pos;
            const { tr } = state;
            if (tabNodes.length <= 1) {
                tr.delete(tabsParentNode.pos, tabsParentNode.pos + tabsParentNode.node.nodeSize);
            }
            else {
                const newTabIdx = tabToRemoveIdx - 1 < 0 ? 1 : tabToRemoveIdx - 1;
                // Change relative pos to absolute
                tabNodes.forEach((v) => {
                    v.pos = v.pos + tabsParentNode.pos + 2;
                });
                const newTabNode = tabNodes[newTabIdx];
                const newTabPanelNode = tabPanels[newTabIdx];
                // Change relative pos to absolute
                newTabPanelNode.pos = newTabPanelNode.pos + tabsParentNode.pos + 1;
                // Find all active tabs and make them inactive
                const activeTabs = tabNodes.filter((v) => v.node.attrs['class'] === const_1.tabActiveClassname);
                if (activeTabs.length) {
                    activeTabs.forEach((tab) => {
                        tr.setNodeMarkup(tab.pos, null, Object.assign(Object.assign({}, tab.node.attrs), { class: const_1.tabInactiveClassname }));
                    });
                }
                // Find all active panels and make them inactive
                const activePanels = tabPanels.filter((v) => v.node.attrs['class'] === const_1.tabPanelActiveClassname);
                if (activePanels.length) {
                    activePanels.forEach((tabPanel) => {
                        tr.setNodeMarkup(tr.mapping.map(tabPanel.pos + tabsParentNode.pos + 1), null, Object.assign(Object.assign({}, tabPanel.node.attrs), { class: const_1.tabPanelInactiveClassname }));
                    });
                }
                tr
                    // Delete panel
                    .delete(panelToRemove.pos, panelToRemove.pos + panelToRemove.node.nodeSize)
                    // Delete tab
                    .delete(tabToRemove.pos, tabToRemove.pos + tabToRemove.node.nodeSize)
                    // Set new active tab
                    .setNodeMarkup(tr.mapping.map(newTabNode.pos), null, Object.assign(Object.assign({}, newTabNode.node.attrs), { class: const_1.tabActiveClassname }))
                    // Set new active panel
                    .setNodeMarkup(tr.mapping.map(newTabPanelNode.pos), null, Object.assign(Object.assign({}, newTabPanelNode.node.attrs), { class: const_1.tabPanelActiveClassname }))
                    .setSelection(prosemirror_state_1.TextSelection.create(tr.doc, tr.mapping.map(newTabNode.pos + newTabNode.node.nodeSize - 1)));
            }
            dispatch(tr);
            return true;
        }
    }
    return false;
};
exports.tabBackspace = tabBackspace;
